#!/usr/local/bin/ruby
=begin
 Signature method Sample
	Copyright (c) 1999 Masatoshi SEKI
	$id:$
=end

=begin
 How to play.
 
 1. Make index file.
 % find ~/Mail/inbox -name '[0-9]*' -print | findmh.rb -m > 000.idx

 2. Retrieve.
 % findmh.rb 000.idx foo bar hoge
=end

require 'nkf'
require 'sigidx'

class MH2Text
  def initialize(fname)
    @fname = fname
  end

  def each_word
    self.each_line do |l|
      l.split.each do |w|
	yield(w)
      end
    end
  end

  def each_line
    begin
      fp = File.open(@fname, "r")
      header = true
      while (s = fp.gets) 
	s.chomp!
	if header
	  if ((s =~ /^From:/) || (s =~ /^Subject:/) || 
	      (s =~ /^To:/) || (s =~ /^Cc:/))
	    yield(NKF.nkf('-me', s))
	  end
	  if s.length <= 0
	    header = false
	  end
	else
	  yield(NKF.nkf('-e', s))
	end
      end
    ensure
      fp.close if fp
    end
  end

  def to_s
    a = []
    each_line do |l| a.push(l) end
    a.join("\n")
  end
end

class SigFind
  def initialize(idxname)
    @idxname = idxname
  end

  def check_file(fname, words)
    text = MH2Text.new(fname).to_s
    words.collect do |w|
      idx = text.index(w)
      return nil unless idx
      idx
    end
  end

  def query(list)
    words = []
    wordsig = SigIdx.new
    list.each do |arg|
      word = NKF.nkf('-e', arg)
      raise 'keyword: too short' if word.size < 3 
      words.push word
      wordsig.add!(word)
    end

    begin
      a = []
      idxf = File.open(@idxname, "r")
      while (s = idxf.gets)
	fname, sig = s.chomp.split
	next unless wordsig.included?(sig)
	offsets = check_file(fname, words)
	next unless offsets
	if iterator?
	  yield fname 
	else
	  a.push fname
	end
      end
    ensure
      idxf.close if idxf
    end
    a
  end
end

class MakeIndex
  def build(ls, out)
    while (s = ls.gets)
      out.print(make_index(s.split.shift))
    end
  end

  def make_index(fname)
    text = MH2Text.new(fname)
    sign = SigIdx.new
    text.each_word do |w|
      sign.add!(w)
    end
    "#{fname}\t#{sign.to_s}\n"
  end
end

if __FILE__ == $0
  def usage
    puts 'findmh.rb <index_file> <keyword> [...]'
    puts 'findmh.rb -m'
    exit 1
  end

  usage if ARGV.size == 0
  
  if ARGV[0] == '-m'
    builder = MakeIndex.new
    builder.build($stdin, $stdout)
  else
    idx = ARGV.shift or usage
    usage if ARGV.size <= 0

    finder = SigFind.new(idx)
    finder.query(ARGV) do |f|
      puts f
    end
  end
end
