/*
 sigidx.c
 $Author: mas $
 $Revision: 1.3 $
*/

#include "ruby.h"
#include "sign.h"

static VALUE
f_sigidx_calc(int argc,
	      VALUE *argv,
	      VALUE obj)
{
    long sigbytes;
    char *s;
    VALUE sign;
    VALUE str;
    VALUE num;

    if (rb_scan_args(argc, argv, "11", &str, &num) == 1) {
	sigbytes = 256;
    } else if (NIL_P(num)) {
	sigbytes = 256;
    } else {
	sigbytes = NUM2INT(num);
    }

    Check_SafeStr(str);
    if (sigbytes <= 0) 
	rb_raise(rb_eRuntimeError, "invalid parameter:sigbytes > 0");

    sign = rb_str_new2("");
    sign = rb_str_resize(sign, sigbytes);

    SignCalc(RSTRING(str)->ptr, RSTRING(sign)->ptr, sigbytes);
    
    return sign;
}

static VALUE
f_sigidx_add(int argc,
	     VALUE *argv,
	     VALUE obj)
{
    long sigbytes;
    char *s;
    VALUE str;
    VALUE sign;
    VALUE num;

    if (rb_scan_args(argc, argv, "21", &sign, &str, &num) == 2) {
	sigbytes = 256;
    } else if (NIL_P(num)) {
	sigbytes = 256;
    } else {
	sigbytes = NUM2INT(num);
    }

    Check_SafeStr(str);
    Check_SafeStr(sign);
    if (sigbytes <= 0) 
	rb_raise(rb_eRuntimeError, "invalid parameter:sigbytes > 0");
    if (sigbytes > RSTRING(sign)->len)
	rb_raise(rb_eRuntimeError, "invalid parameter:sigbytes <= sign.size");
	
    SignAdd(RSTRING(str)->ptr, RSTRING(sign)->ptr, sigbytes);
    
    return sign;
}

static VALUE
f_sigidx_match(int argc,
	       VALUE *argv,
	       VALUE obj)
{
    long sigbytes;
    VALUE sign;
    int match;
    VALUE textsig;
    VALUE patsig;
    VALUE num;

    if (rb_scan_args(argc, argv, "21", &textsig, &patsig, &num) == 2) {
	sigbytes = 256;
    } else if (NIL_P(num)) {
	sigbytes = 256;
    } else {
	sigbytes = NUM2INT(num);
    }

    Check_SafeStr(textsig);
    Check_SafeStr(patsig);
    if (sigbytes <= 0) 
	rb_raise(rb_eRuntimeError, "invalid parameter:sigbytes > 0");
    if (sigbytes > RSTRING(textsig)->len || sigbytes > RSTRING(patsig)->len) 
	rb_raise(rb_eRuntimeError, "signature is too short.");

    match = SignMatch(RSTRING(textsig)->ptr, RSTRING(patsig)->ptr, sigbytes);

    return match ? Qtrue : Qfalse;
}

VALUE cSigIdx;

void
Init_sigidx() {
    cSigIdx = rb_define_class("SigIdx", rb_cObject);
    rb_define_module_function(cSigIdx, "calc", f_sigidx_calc, -1);
    rb_define_module_function(cSigIdx, "add", f_sigidx_add, -1);
    rb_define_module_function(cSigIdx, "match", f_sigidx_match, -1);
    rb_eval_string("class SigIdx\n"
		   "def initialize(sig=nil, sz=256)\n"
		   "  @size = sz\n"
		   "  if sig\n"
		   "    @sign = sig.to_s\n"
		   "  else\n"
		   "    @sign = SigIdx.calc('', @size)\n"
		   "  end\n"
		   "end\n"
		   "attr :size\n"
		   "attr :sign\n"
		   "def to_s; @sign; end\n"
		   "def add!(s); SigIdx.add(@sign, s.to_s, @size); end\n"
		   "def include?(sig)\n"
		   "  SigIdx.match(@sign, sig.to_s, @size)\n"
		   "end\n"
		   "def included?(sig)\n"
		   "  SigIdx.match(sig.to_s, @sign, @size)\n"
		   "end\n"
		   "end\n");
}
