#include "sign.h"

static int hashpjw(unsigned char const *s,
		   int len)
{
    unsigned int h = 0;
    unsigned int g;

    for (; *s && (len-- > 0); s++) {
	h = (h << 4) + (*s);
	if (g = h & 0xf0000000) {
	    h = h ^ (g >> 24);
	    h = h ^ g;
	}
    }
    return h;
}

void SignCalc(unsigned char const *s,
	      char *sig,
	      int sigbytes)
{
    int sigbits = sigbytes * 6;
    int i;
    int v;

    for (i = 0; i < sigbytes; i++) 
	sig[i] = 0x40;
    sig[sigbytes] = 0;
    SignAdd(s, sig, sigbytes);
}

void SignAdd(unsigned char const *s,
	     char *sig,
	     int sigbytes)
{
    int sigbits = sigbytes * 6;
    int i;
    int v;

    for (; s[0] && s[1] && s[2]; s++) {
	v = hashpjw(s, 3) % sigbits;
	sig[v / 6] |= (1 << v % 6);
    }
    for (i = 0; i < sigbytes; i++)
	if (sig[i] == 0x7f)
	    sig[i] = 0x3f;
}


int SignMatch(char const *textsig,
	      char const *patsig,
	      int sigbytes)
{
    int sigbits = sigbytes * 6;
    int i;
    char c;
    
    for (i = 0; i < sigbytes; i++) {
	c = patsig[i] & 0x3f;
	if ((textsig[i] & c) != c)
	    return 0;
    }
    return 1;
}


void SignOr(char const *sig1,
	    char const *sig2,
	    char *sign,
	    int sigbytes)
{
    int i;
    char c;

    for (i = 0; i < sigbytes; i++) {
	c = 0x40 | (sig1[i] & 0x3f) | (sig2[i] & 0x3f);
	sign[i] = (c == 0x7f) ? 0x3f : c;
    }
}
